# Load the presets
if(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
  include(osx.preset)
else(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
  if(NOT CMAKE_CROSSCOMPILING)
    include(linux.preset)
  endif(NOT CMAKE_CROSSCOMPILING)
endif(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")

# Main Settings

set(WITH_SYSTEM_CALLS "none" CACHE STRING "Choose what system calls can be used, options are: none, sbrk, mmap")
set(SYS_ALLOC_SIZE 4096 CACHE INTEGER "Choose the default system allocation size")

option(HAVE_LOCKS "Build with POSIX locking mechanisms" OFF)
option(TRYLOCK_ON_MALLOC "Use a trylock on malloc()" OFF)

set(RAW_BLOCKS_TYPE "freelist" CACHE STRING "Choose what raw blocks can be used, options are: freelist, bitmap")

# Free-list Settings
set(ORDERING_POLICY "lifo" CACHE STRING "Choose the ordering policy in freelist-organized raw blocks, options are: address, fifo, lifo, size")
set(FITTING_POLICY "best" CACHE STRING "Choose the fitting policy in freelist-organized raw blocks, options are: best, exact, first, good")
set(GOOD_FIT_PERCENTAGE 0.8 CACHE DOUBLE "Choose the good-fit percentage")
set(WITH_COALESCING "never" CACHE STRING "Build with coalescing support")
set(WITH_SPLITTING "never" CACHE STRING "Build with splitting support")
option(FREELIST_COALESCE_AFTER_SPLIT "Try to coalesce blocks after split" OFF)

# Bitmap Settings
set(BITMAP_RESOLUTION 256 CACHE INTEGER "Choose the size of cells in bitmap-organised raw blocks")

# Function Implementation Settings

option(WITH_REALLOC "Build with realloc() support" ON)
option(WITH_CALLOC "Build with calloc() support" ON)
option(WITH_MEMALIGN "Build with memalign() support" ON)

# GLIBC Hooks Settings

option(WITH_GLIBC_HOOKS "Place dmmlib functions on GLIBC hooks" OFF)

# Knobs Settings

option(WITH_KNOBS "Build with knobs support" OFF)

# Statistics Settings

set(STATS "none" CACHE STRING "Choose if the memory allocator keeps internally statistics per raw block or globally, options are: none, global")
option(REQUEST_SIZE_INFO "Keep request size information in metadata" OFF)

# Trace Settings

option(WITH_MEM_TRACE "Support for memory traces" OFF)
option(WITH_STATS_TRACE "Support for statistics traces" OFF)

# Build Settings

option(WITH_STATIC_LIB "Build a static library" OFF)
option(WITH_SHARED_LIB "Build a shared library" OFF)

# Misc Settings

option(PARSE_ENV "Build with support to parse dmmlib environment variables" OFF)
option(WITH_EXAMPLES "Build with examples" OFF)
option(WITH_DOC "Build with documentation" OFF)
